#! /usr/bin/env python3
###############################################################################
#                                                                             #
# GALORE: Gaussian and Lorentzian broadening for simulated spectra            #
#                                                                             #
# Developed by Adam J. Jackson and Alex Ganose                                #
# at University College London (2017)                                         #
#                                                                             #
###############################################################################
#                                                                             #
# This file is part of Galore. Galore is free software: you can redistribute  #
# it and/or modify it under the terms of the GNU General Public License as    #
# published by the Free Software Foundation, either version 3 of the License, #
# or (at your option) any later version.  This program is distributed in the  #
# hope that it will be useful, but WITHOUT ANY WARRANTY; without even the     #
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    #
# See the GNU General Public License for more details.  You should have       #
# received a copy of the GNU General Public License along with this program.  #
# If not, see <http://www.gnu.org/licenses/>.                                 #
#                                                                             #
###############################################################################
from __future__ import print_function, absolute_import, division

import os
import argparse
from collections import OrderedDict
from json import load as json_load
import logging
import warnings

import numpy as np

import galore
import galore.formats
import galore.plot
from galore import auto_limits

try:
    import matplotlib
    has_matplotlib = True
except ImportError:
    has_matplotlib = False


def main():
    logging.basicConfig(filename='galore.log', level=logging.INFO)
    console = logging.StreamHandler()
    logging.getLogger().addHandler(console)

    warnings.filterwarnings("ignore", module="matplotlib")
    warnings.filterwarnings("ignore", module="pymatgen")

    parser = get_parser()
    args = parser.parse_args()
    args = vars(args)
    run(**args)


def run(**kwargs):
    if kwargs['sampling']:
        pass
    elif kwargs['units'] in ('cm', 'cm-1'):
        kwargs['sampling'] = 0.1
    elif kwargs['units'] in ('THz', 'thz'):
        kwargs['sampling'] = 1e-3
    elif kwargs['units'] in ('ev', 'eV'):
        kwargs['sampling'] = 1e-2
    else:
        kwargs['sampling'] = 1e-2

    if kwargs['pdos']:
        pdos_from_files(**kwargs)
    else:
        simple_dos_from_files(**kwargs)


def pdos_from_files(return_plt=False, **kwargs):
    """Read input data, process for PDOS before plotting and/or writing

    Args:
        return_plt (bool): If True, return the pyplot object instead of writing
            or displaying plot output.
        **kwargs: See command reference for full argument list

    """
    pdos_plotting_data = galore.process_pdos(**kwargs)

    # For plotting and writing, "None" means "write to screen"
    # while False means "do nothing"
    if kwargs['plot'] or kwargs['plot'] is None:
        plt = galore.plot.plot_pdos(pdos_plotting_data,
                                    **kwargs)  # flipx is included in kwargs

        if kwargs['overlay'] is not None:
            plt = galore.plot.add_overlay(
                plt, kwargs['overlay'],
                overlay_offset=kwargs['overlay_offset'],
                overlay_scale=kwargs['overlay_scale'],
                overlay_style=kwargs['overlay_style'],
                overlay_label=kwargs['overlay_label'])
            plt.legend(loc='best')

        xlabel = galore.plot.guess_xlabel(units=kwargs['units'],
                                          flipx=kwargs['flipx'],
                                          energy_label=None)
        plt.xlabel(xlabel)

        if kwargs['ylabel'] is not None:
            plt.ylabel(kwargs['ylabel'])

        if return_plt:
            return plt
        elif kwargs['plot']:
            plt.savefig(kwargs['plot'])
        elif kwargs['plot'] is None:
            plt.show()

    if kwargs['csv'] or kwargs['csv'] is None:
        galore.formats.write_pdos(pdos_plotting_data,
                                  filename=kwargs['csv'],
                                  filetype='csv',
                                  flipx=kwargs['flipx'])

    if kwargs['txt'] or kwargs['txt'] is None:
        galore.formats.write_pdos(pdos_plotting_data,
                                  filename=kwargs['txt'],
                                  filetype='txt',
                                  flipx=kwargs['flipx'])


def simple_dos_from_files(return_plt=False, **kwargs):
    """Generate a spectrum or DOS over one data series

    kwargs['input'] can be a string or a list containing one string.
    In addition to main kwargs documented for CLI

    Args:
        return_plt (bool): If True, return the pyplot object instead of writing
            or displaying plot output.
        **kwargs: See command reference for full argument list

    """

    x_values, broadened_data = galore.process_1d_data(**kwargs)

    if not any(((kwargs['csv'] is None), (kwargs['txt'] is None),
                (kwargs['plot'] is None),
                kwargs['csv'], kwargs['txt'], kwargs['plot'])):
        print("No output selected. Please use at least one of the output "
              "options (CSV, txt, plotting). For usage information, run "
              "galore with -h argument.")

    if kwargs['plot'] or kwargs['plot'] is None:
        if not has_matplotlib:
            print("Can't plot, no Matplotlib")
        else:
            plt = galore.plot.plot_tdos(x_values, broadened_data, **kwargs)
            if kwargs['ylabel'] is not None:
                plt.ylabel(kwargs['ylabel'])
            if return_plt:
                return plt
            elif kwargs['plot']:
                plt.savefig(kwargs['plot'])
            else:
                plt.show()

    if kwargs['csv'] is None:
        galore.formats.write_csv(x_values, broadened_data, filename=None)
    elif kwargs['csv']:
        galore.formats.write_csv(
            x_values, broadened_data, filename=kwargs['csv'])

    if kwargs['txt'] is None:
        galore.formats.write_txt(x_values, broadened_data, filename=None)
    elif kwargs['txt']:
        galore.formats.write_txt(
            x_values, broadened_data, filename=kwargs['txt'])


def get_parser():
    """Parse command-line arguments. Function is used to build the CLI docs."""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        'input', type=str, default='vasprun.xml', nargs='+',
        help='Input data file. The supported formats are vasprun.xml (VASP '
             'output), *.gpw (GPAW output), *.csv (comma-delimited text) '
             'and *.txt (space-delimited text).')
    parser.add_argument(
        '-l',
        '--lorentzian',
        nargs='?',
        default=False,
        const=2,
        type=float,
        help='Apply Lorentzian broadening with specified width.')
    parser.add_argument(
        '-g',
        '--gaussian',
        nargs='?',
        default=False,
        const=2,
        type=float,
        help='Apply Gaussian broadening with specified width.')
    parser.add_argument(
        '-w', '--weighting',
        type=str,
        default=None,
        help='Apply cross-section weighting to data. "alka", "he2" and '
             '"yeh_haxpes" select tabulated data for valence band at 1486.6 '
             'eV, 40.8 eV and 8047.8 eV respectively. '
             'Numerical values will be interpreted as an energy in keV; '
             'for energies from 1-1500 eV cross-sections will be determined '
             'using a parametrisation from tabulated data. '
             'Alternatively, provide '
             'path to a JSON file with cross-section data.')
    parser.add_argument(
        '--units',
        '--x_units',
        type=str,
        default='',
        choices=('cm', 'cm-1', 'thz', 'THz',
                 'ev', 'eV', 'ry', 'Ry', 'ha', 'Ha'),
        help='Units for x axis (usually frequency or energy)')
    parser.add_argument(
        '--ylabel',
        type=str,
        default=None,
        help='Label for plot y-axis')
    parser.add_argument(
        '--txt',
        nargs='?',
        default=False,
        const=None,
        help='Write broadened output as space-delimited text; file if path '
             'provided, otherwise write to standard output.')
    parser.add_argument(
        '--csv',
        nargs='?',
        default=False,
        const=None,
        help='Write broadened output as comma-separated values; file if path '
             'provided, otherwise write to standard output.')
    parser.add_argument(
        '-p',
        '--plot',
        nargs='?',
        default=False,
        const=None,
        help='Plot broadened spectrum. Plot to filename if provided,'
             ' otherwise display to screen.')
    parser.add_argument(
        '-d',
        '--sampling',
        type=float,
        default=False,
        help='Width, in units of x, of x-axis resolution. If not specified, '
             'default value is based on units. If units are not specified, '
             'default value is 1e-2.')
    parser.add_argument(
        '-k',
        '--spikes',
        '--spike',
        action='store_true',
        help='Resample data as "spikes" on a zero baseline. The default is to '
             'interpolate linearly between y-values, which is reasonable for '
             'distributions such as DOS. If the input data set only contains '
             'active energies/frequencies (e.g. IR modes) then you should use '
             '--spike mode. See tutorials for examples.')
    parser.add_argument(
        '--pdos', action="store_true", help='Use orbital-projected data')
    parser.add_argument(
        '--flipx', '--xflip', action="store_true",
        help='Negate x-values in output; this may be helpful for comparison '
             'with binding energy measurments.')
    parser.add_argument(
        '--xmin', type=float, default=None, help='Minimum x axis value')
    parser.add_argument(
        '--xmax', type=float, default=None, help='Maximum x axis value')
    parser.add_argument(
        '--ymin', type=float, default=0, help='Minimum y axis value')
    parser.add_argument(
        '--ymax', type=float, default=None, help='Maximum y axis value')
    parser.add_argument(
        '--overlay', type=str, default=None, help='Data file for overlay')
    parser.add_argument(
        '--overlay_scale', type=float, default=None,
        help='Y-axis scale factor for data overlay')
    parser.add_argument(
        '--overlay_offset', type=float, default=0,
        help='X-axis offset for data overlay')
    parser.add_argument(
        '--overlay_style', type=str, default='o',
        help='Matplotlib line style for overlay data. Default "o" for '
             'circles, "x:" for crosses joined by dotted lines, etc.')
    parser.add_argument(
        '--overlay_label', type=str, default=None,
        help='Legend label for data overlay'
        )

    return parser

if __name__ == '__main__':
    main()



from __future__ import division, absolute_import

import os
from os.path import join as path_join
import sys
import unittest
import shutil
import tempfile

import numpy.testing
from numpy.testing import assert_array_equal
import numpy as np

import galore
import galore.formats
from galore.cli.galore import simple_dos_from_files
import galore.plot

from contextlib import contextmanager
import io

test_dir = os.path.abspath(os.path.dirname(__file__))

try:
    import pymatgen
    has_pymatgen = True
except ImportError:
    has_pymatgen = False


@contextmanager
def stdout_redirect():
    """Enable tests to inspect stdout in suitable format for Python version"""
    if sys.version_info > (3,):
        output = io.StringIO()
    else:
        output = io.BytesIO()
    sys.stdout = output
    try:
        yield output
    finally:
        output.close()


class test_dos_functions(unittest.TestCase):
    def test_simple_dos_spikes(self):
        """Test total DOS / spectrum plotter from CSV data, spike sampling"""
        ylabel = 'some label'
        xmin = -3
        xmax = 220
        sampling = 1e-1
        plt = simple_dos_from_files(input=path_join(test_dir,
                                                    'test_xy_data.csv'),
                                    return_plt=True, xmax=xmax, xmin=xmin,
                                    sampling=sampling,
                                    spikes=True,
                                    lorentzian=2.3, gaussian=3.2,
                                    csv=False, txt=False, plot=None,
                                    units='cm-1', ymax=None, ymin=None,
                                    ylabel=ylabel,
                                    flipx=False)
        fig = plt.gcf()
        ax = fig.axes[0]
        self.assertEqual(ax.get_ylabel(), ylabel)
        self.assertEqual(ax.get_xlabel(), r'cm$^{-1}$')
        self.assertAlmostEqual(ax.get_xlim()[0], xmin, places=2)
        self.assertLess(ax.get_xlim()[1], xmax)
        self.assertGreater(ax.get_xlim()[1], (xmax * 0.99))
        self.assertEqual(len(ax.lines), 1)
        xvals, yvals = ax.lines[0].get_xydata().T
        self.assertAlmostEqual(xvals[5], (xmin + 5 * sampling))
        self.assertAlmostEqual(yvals[5], 0.0, places=3)
        self.assertAlmostEqual(yvals[2000], 0.65245445, places=4)

    def test_simple_dos_linear(self):
        """Test total DOS / spectrum plotter from CSV data, linear sampling"""
        ylabel = 'some label'
        xmin = -3
        xmax = 220
        sampling = 1e-1
        plt = simple_dos_from_files(input=path_join(test_dir,
                                                    'test_xy_data.csv'),
                                    return_plt=True, xmax=xmax, xmin=xmin,
                                    sampling=sampling,
                                    lorentzian=2.3, gaussian=3.2,
                                    csv=False, txt=False, plot=None,
                                    units='cm-1', ymax=None, ymin=None,
                                    ylabel=ylabel,
                                    flipx=False)
        fig = plt.gcf()
        ax = fig.axes[0]
        self.assertEqual(ax.get_ylabel(), ylabel)
        self.assertEqual(ax.get_xlabel(), r'cm$^{-1}$')
        self.assertAlmostEqual(ax.get_xlim()[0], xmin, places=2)
        self.assertLess(ax.get_xlim()[1], xmax)
        self.assertGreater(ax.get_xlim()[1], (xmax * 0.99))
        self.assertEqual(len(ax.lines), 1)
        xvals, yvals = ax.lines[0].get_xydata().T
        self.assertAlmostEqual(xvals[5], (xmin + 5 * sampling))
        self.assertAlmostEqual(yvals[5], 0.0, places=3)
        self.assertAlmostEqual(yvals[2000], 98.64411, places=4)

    def test_overlay(self):
        import matplotlib.pyplot as plt
        fig = plt.figure()
        ax = fig.add_subplot(1, 1, 1)
        galore.plot.add_overlay(plt, path_join(test_dir, 'test_xy_data.csv'),
                                overlay_scale=2, overlay_offset=1,
                                overlay_style="x:", overlay_label="foo")
        ax = plt.gca()
        line = ax.lines[0]
        xy = line.get_xydata()
        self.assertEqual(xy[3, 0], 201)
        self.assertEqual(xy[3, 1], 0.6)
        self.assertEqual(xy.shape, (8, 2))


class test_array_functions(unittest.TestCase):
    def test_delta(self):
        self.assertEqual(galore.delta(1, 1.5, w=1), 1)

    def test_xy_to_1d_spikes(self):
        """Check resampling of distinct values as spikes"""
        assert_array_equal(
            galore.xy_to_1d(
                np.array([[2.1, 0.6], [4.3, 0.2], [5.1, 0.3]]), range(6),
                         spikes=True),
            np.array([0., 0., 0.6, 0., 0.2, 0.3]))
    def test_xy_to_1d_linear(self):
        """Check resampling with linear interpolation"""
        assert_array_equal(
            galore.xy_to_1d(
                np.array([[1., 0.5], [3., 1.5]]), range(6),
                         spikes=False),
            np.array([0., 0.5, 1.0, 1.5, 0., 0.0]))


    def test_gaussian(self):
        self.assertAlmostEqual(galore.gaussian(3., f0=1, fwhm=(3 * 2.35482)),
                               0.8007374029168)


class test_io_functions(unittest.TestCase):
    def setUp(self):
        self.tempdir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tempdir)

    def test_identify_raman(self):
        doscar_path = path_join(test_dir, 'DOSCAR.1')
        raman_path = path_join(test_dir, 'CaF2', 'raman_lda_500.dat')
        self.assertFalse(galore.formats.is_vasp_raman(doscar_path))
        self.assertTrue(galore.formats.is_vasp_raman(raman_path))

    def test_identify_doscar(self):
        doscar_path = path_join(test_dir, 'DOSCAR.1')
        raman_path = path_join(test_dir, 'CaF2', 'raman_lda_500.dat')
        self.assertTrue(galore.formats.is_doscar(doscar_path))
        self.assertFalse(galore.formats.is_doscar(raman_path))

    def test_write_txt(self):
        x_values = range(5)
        y_values = [x**2 / 200 for x in range(5)]
        filename = path_join(self.tempdir, 'write_txt_test.txt')
        galore.formats.write_txt(
            x_values, y_values, filename=filename, header="# Frequency  Value")
        with open(filename, 'r') as f:
            self.assertEqual(f.read(), txt_test_string)

    def test_write_txt_stdout(self):
        with stdout_redirect() as stdout:
            x_values = range(5)
            y_values = [x**2 / 200 for x in range(5)]
            filename = path_join(self.tempdir, 'write_txt_test.txt')
            galore.formats.write_txt(
                x_values, y_values, filename=None, header="# Frequency  Value")
            self.assertEqual(stdout.getvalue(), txt_test_string)

    def test_write_csv(self):
        x_values = range(5)
        y_values = [x**2 / 200 for x in range(5)]
        filename = path_join(self.tempdir, 'write_csv_test.csv')
        galore.formats.write_csv(
            x_values,
            y_values,
            filename=filename,
            header=["Frequency", "Value"])
        with open(filename, 'r') as f:
            self.assertEqual(f.read(), csv_test_string)

    def test_write_csv_stdout(self):
        with stdout_redirect() as stdout:
            x_values = range(5)
            y_values = [x**2 / 200 for x in range(5)]
            galore.formats.write_csv(
                x_values,
                y_values,
                filename=None,
                header=["Frequency", "Value"])
            self.assertEqual(stdout.getvalue(), csv_test_string)

    def test_read_spinpol_doscar(self):
        doscar_path = path_join(test_dir, 'DOSCAR.1')
        data = galore.formats.read_doscar(doscar_path)
        self.assertEqual(data[20, 0], -31.795)
        self.assertEqual(data[14, 1], 0.329)

    def test_read_raman(self):
        raman_path = path_join(test_dir, 'CaF2', 'raman_lda_500.dat')
        raman_data = np.array([[3.45589820e+02, 9.89999400e-01],
                               [3.45589690e+02, 9.89999400e-01],
                               [3.45580570e+02, 9.89999400e-01],
                               [2.78757900e+02, 0.00000000e+00],
                               [2.78757810e+02, 0.00000000e+00],
                               [2.78757760e+02, 1.00000000e-07],
                               [6.11230000e-01, 0.00000000e+00],
                               [6.11260000e-01, 0.00000000e+00],
                               [6.11920000e-01, 3.80000000e-06]])
        assert_array_equal(galore.formats.read_vasp_raman(raman_path),
                           raman_data)

    @unittest.skipUnless(has_pymatgen, "requires pymatgen")
    def test_read_vasprun_totaldos(self):
        vr_path = path_join(test_dir, 'MgO', 'vasprun.xml.gz')
        data = galore.formats.read_vasprun_totaldos(vr_path)
        self.assertEqual(data[150, 0], -17.2715)
        self.assertEqual(data[195, 1], 16.8066)

    @unittest.skipUnless(has_pymatgen, "requires pymatgen")
    def test_read_vasprun_pdos(self):
        vr_path = path_join(test_dir, 'MgO', 'vasprun.xml.gz')
        pdos = galore.formats.read_vasprun_pdos(vr_path)
        self.assertEqual(pdos['Mg']['s'][150], 0.053)
        self.assertEqual(pdos['O']['p'][189], 0.004)


txt_test_string = """# Frequency  Value
0.000000e+00 0.000000e+00
1.000000e+00 5.000000e-03
2.000000e+00 2.000000e-02
3.000000e+00 4.500000e-02
4.000000e+00 8.000000e-02
"""

csv_test_string = os.linesep.join(
    ("Frequency,Value", "0,0.0", "1,0.005", "2,0.02", "3,0.045", "4,0.08", ""))

if __name__ == '__main__':
    unittest.main()



import unittest
import sys
import io
from os.path import join as path_join
from pkg_resources import resource_filename
from galore.formats import read_gpaw_totaldos, read_gpaw_pdos
from contextlib import contextmanager

# try:
#     from gpaw import GPAW
#     import ase.build
#     has_gpaw = True
# except ImportError:
#     has_gpaw = False
from gpaw import GPAW
import ase.build

@contextmanager
def stdout_redirect():
    """Enable tests to inspect stdout in suitable format for Python version"""
    if sys.version_info > (3,):
        output = io.StringIO()
    else:
        output = io.BytesIO()
    sys.stdout = output
    try:
        yield output
    finally:
        output.close()


# @unittest.skipIf(not has_gpaw, "GPAW not available")
class TestGPAW(unittest.TestCase):
    gpaw_file = resource_filename(__name__, path_join('CdTe', 'CdTe.gpw'))

    ref_vbm = 4.948260
    efermi = 5.0761385

    def test_tdos_read(self):
        """Check TDOS can be read from GPAW file"""
        with stdout_redirect() as stdout:
            tdos = read_gpaw_totaldos(self.gpaw_file, npts=1000)
        self.assertEqual(tdos.shape, (1000, 2))
        self.assertAlmostEqual(tdos[-1, 0], 4.3982112, places=6)
        self.assertAlmostEqual(tdos[-1, 1], 1.5670867e-8, places=6)

    def test_energy_refs(self):
        """Check alternative GPAW energy references"""
        with stdout_redirect() as stdout:
            calc = GPAW(self.gpaw_file)
        self.assertAlmostEqual(calc.get_homo_lumo()[0], self.ref_vbm, places=6,
                               msg="VBM from GPAW doesn't match reference")
        self.assertAlmostEqual(calc.get_fermi_level(), self.efermi, places=6,
                               msg="E-Fermi from GPAW doesn't match reference")

    def test_tdos_refs(self):
        """Check GPAW TDOS energy alignment options"""
        with stdout_redirect() as stdout:
            tdos_0 = read_gpaw_totaldos(self.gpaw_file, npts=1000, ref=None)
            tdos_vbm = read_gpaw_totaldos(self.gpaw_file, npts=1000, ref='vbm')
            tdos_efermi = read_gpaw_totaldos(self.gpaw_file, npts=1000,
                                             ref='efermi')

        self.assertAlmostEqual(tdos_vbm[0, 0], tdos_0[0, 0] - self.ref_vbm)
        self.assertAlmostEqual(tdos_efermi[0, 0], tdos_0[0, 0] - self.efermi)

    def test_pdos_read(self):
        with stdout_redirect() as stdout:
            pdos = read_gpaw_pdos(self.gpaw_file, npts=1000)
        for species in ('Cd', 'Te'):
            self.assertIn(species, pdos)
        for column in ('s', 'p', 'd', 'f', 'energy'):
            self.assertIn(column, pdos['Cd'])
        for species in ('Cd', 'Te'):
            self.assertIsNone(pdos[species]['f'])
        self.assertAlmostEqual(max(pdos['Cd']['s']), 0.00295165, places=6,
                               msg="PDOS value doesn't match reference")
        self.assertAlmostEqual(pdos['Cd']['energy'][100], -9.5542409,
                               places=6,
                               msg="PDOS energy value doesn't match reference")
        self.assertTrue((pdos['Cd']['energy'] == pdos['Te']['energy']).all(),
                        msg="PDOS energy ranges not consistent")

    def test_pdos_refs(self):
        """Check GPAW PDOS energy alignment options"""
        with stdout_redirect() as stdout:
            pdos_0 = read_gpaw_pdos(self.gpaw_file,
                                    npts=1000, ref=None)
            pdos_vbm = read_gpaw_pdos(self.gpaw_file,
                                      npts=1000, ref='vbm')
            pdos_efermi = read_gpaw_pdos(self.gpaw_file,
                                         npts=1000, ref='efermi')

        self.assertAlmostEqual(pdos_vbm['Te']['energy'][0],
                               pdos_0['Te']['energy'][0] - self.ref_vbm)
        self.assertAlmostEqual(pdos_efermi['Te']['energy'][0],
                               pdos_0['Te']['energy'][0] - self.efermi)



import os.path
from pkg_resources import resource_filename
from json import load as json_load
from collections import Iterable

import sqlite3
from scipy import polyval
from numpy import fromstring as np_fromstr
from numpy import exp, log


def get_cross_sections(weighting, elements=None):
    """Get photoionization cross-section weighting data.

    For known sources, data is based on tabulation of Yeh/Lindau (1985).[1]
    Otherwise, energies in keV from 1-1500 are used with log-log polynomial
    parametrisation of data from Scofield.[2]

    References:
      1. Yeh, J.J. and Lindau, I. (1985)
         Atomic Data and Nuclear Data Tables 32 pp 1-155
      2. J. H. Scofield (1973) Lawrence Livermore National Laboratory
         Report No. UCRL-51326

    Args:
        weighting (str or float): Data source for photoionization
            cross-sections. If the string is a known keyword then data will
            be drawn from files included with Galore. Otherwise, the string
            will be interpreted as a path to a JSON file containing data
            arranged in the same way as the output of this function.
        elements (iterable or None): Collection of element symbols to include
            in the data set. If None, a full set of available elements will be
            included. When using a JSON dataset (including the inbuilt
            Yeh/Lindau) this parameter will be ignored as the entire dataset
            has already been loaded into memory.

    Returns:
        dict:
            Photoionization cross-section weightings arranged by element and
            orbital as nested dictionaries of floats, i.e.::

                {el1: {orb1: cs11, orb2: cs12, ...},
                 el2: {orb1: cs21, orb2: cs22, ...}, ... }

            In addition the keys "reference", "link", "energy" and "warning"
            may be used to store metadata.

    """

    try:
        energy = float(weighting)
        return get_cross_sections_scofield(energy, elements=elements)

    except ValueError:
        if isinstance(weighting, str):
            if weighting.lower() in ('alka', 'he2', 'yeh_haxpes'):
                return get_cross_sections_yeh(weighting)
            elif weighting.lower() in ('xps', 'ups', 'haxpes'):
                raise ValueError("Key '{0}' is no longer accepted for "
                                 "weighting. Please use 'alka' for Al k-alpha,"
                                 " 'he2' for He (II) or 'yeh_haxpes' for "
                                 "8047.8 eV HAXPES".format(weighting))
            else:
                return get_cross_sections_json(weighting)

    # A string or a number would have hit a return statement by now
    raise ValueError("Weighting not understood as string or float. ",
                     "Please use a keyword, path to JSON file or an "
                     "energy value in eV")


def cross_sections_info(cross_sections, logging=None):
    """Log basic info from cross-sections dict.

    Args:
        cross_sections (dict): The keys 'energy', 'citation',
            'link' and 'warning' are checked for relevant information

        logging (module): Active logging module from Python standard
            library. If None, logging will be set up.

    Returns:
        module:
            Active logging module from Python standard library

    """

    if logging is None:
        import logging
        logging.basicConfig(filename='galore.log', level=logging.INFO)
        console = logging.StreamHandler()
        logging.getLogger().addHandler(console)

    if 'energy' in cross_sections:
        logging.info("  Photon energy: {0}".format(cross_sections['energy']))
    if 'citation' in cross_sections:
        logging.info("  Citation: {0}".format(cross_sections['citation']))
    if 'link' in cross_sections:
        logging.info("  Link: {0}".format(cross_sections['link']))

    return logging


def get_cross_sections_json(path):
    """Get valence-band cross-sections from JSON file

    Read photoionization data from a JSON file. File is expected to contain
    data for multiple elements and orbitals in the form
    ``{El1: {orb1: c1, orb2: c2, ...}, ...}``. While it is expected that
    Galore will be used to examine valence-band orbitals labelled (s, p, d, f)
    it may be helpful in some cases to prepare a file with alternative orbital
    labels corresponding to the pDOS labels.

    The labels 'citation', 'energy' and 'link' are reserved for metadata which
    may be displayed in the program log.
    The label 'comment' may be used for additional material in the JSON file;
    it is recommended to use this repeatedly for line-breaks, e.g.::

        {"comment": "First line of text",
         "comment": "which is continued.",
         ...}

    Args:
        path (str): Path to JSON file

    Returns:
        dict:
            Weighted photoionization cross-sections for each element and
            orbital in form::

                {el1: {'s': c11, 'p': c12, ... },
                 el2: {'s': c21, 'p': c22, ... }, ... }

            in tabulated units.

    """

    if os.path.exists(path):
        with open(path, 'r') as f:
            cross_sections = json_load(f)
    else:
        raise IOError("Cross-sections file {0} does not "
                      "exist!".format(path))

    return cross_sections


def get_cross_sections_yeh(source):
    """Get valence-band cross-sections from tabulated data

    Tabulated values of photoionization cross-sections were drawn from ref [1]
    for energy values corresponding to relevant radiation sources:
    - 1486.6 eV, corresponding to Al k-alpha (laboratory XPS)
    - 40.8 eV, corresponding to He II (laboratory UPS)
    - 8047.8 eV, corresponding to a possible HAXPES source

    References:
        1. Yeh, J.J. and Lindau, I. (1985)
           Atomic Data and Nuclear Data Tables 32 pp 1-155

    Args:
        source (str): Label corresponding to radiation source. Accepted values
            'alka' (1486.6 eV), 'he2' (40.8 eV), 'yeh_haxpes' (8047.8).
            These keys are not case-sensitive and correspond to Al k-alpha,
            He(II) and hard x-ray sources.

    Returns:
        dict:
            Weighted photoionization cross-sections in megaBarns/electron
            for each orbital in form::

                {el1: {'s': c11, 'p': c12, ... },
                 el2: {'s': c21, 'p': c22, ... }, ... }

    """

    weighting_files = {'alka': resource_filename(
                       __name__, "data/cross_sections.json"),
                       'he2': resource_filename(
                           __name__, "data/cross_sections_ups.json"),
                       'yeh_haxpes': resource_filename(
                           __name__, "data/cross_sections_haxpes.json")}

    if source.lower() in weighting_files:
        path = weighting_files[source.lower()]
        return get_cross_sections_json(path)

    else:
        raise Exception(
            "Energy source '{0}' not recognised. ".format(source),
            "Accepted values: {0}".format(", ".join((weighting_files.keys()))))


def get_cross_sections_scofield(energy, elements=None):
    """Get valence-band cross-sections from fitted data

    Energy-dependent cross-sections have been averaged and weighted for
    the uppermost s, p, d, f orbitals from data tabulated by Scofield.
    The energy/cross-section relationship was fitted to an order-8
    polynomial on a log-log scale.

    Multiple energy values can be evaluated simultaneously by passing an
    array-like group of energies as ``energy``. In this case the cross-section
    values will be arrays with the same shape as the energy arrays.

    Args:
        energy (float or array-like): Incident energy in keV
        element (iterable or None): Iterable (e.g. list) of element symbols. If
            None, include all available elements (1 <= Z <= 100).

    Returns:
        dict:
            Weighted photoionization cross-sections in Barns/electron
            for each orbital in form::

                {el1: {'s': c11, 'p': c12, ... },
                 el2: {'s': c21, 'p': c22, ... }, ... }

    Raises:
        ValueError: Energy values must lie within interpolation range
            1--1500keV

    """

    min_energy, max_energy = 1., 1500.

    def _low_value(energy):
        raise ValueError("Scofield data not available below 1 keV: refusing"
                         " to extrapolate to {0} keV".format(energy))

    def _high_value(energy):
        raise ValueError("Scofield data not available above 1500 keV: refusing"
                         " to extrapolate to {0} keV".format(energy))

    if isinstance(energy, Iterable):
        if min(energy) < min_energy:
            _low_value(energy)
        elif max(energy) > max_energy:
            _high_value(energy)
    else:
        if energy < min_energy:
            _low_value(energy)
        elif energy > max_energy:
            _high_value(energy)

    db_file = resource_filename(__name__, "data/scofield_data.db")

    if elements is None:
        elements = ['H', 'He', 'Li', 'Be', 'B', 'C', 'N', 'O', 'F', 'Ne', 'Na',
                    'Mg', 'Al', 'Si', 'P', 'S', 'Cl', 'Ar', 'K', 'Ca', 'Sc',
                    'Ti', 'V', 'Cr', 'Mn', 'Fe', 'Co', 'Ni', 'Cu', 'Zn', 'Ga',
                    'Ge', 'As', 'Se', 'Br', 'Kr', 'Rb', 'Sr', 'Y', 'Zr', 'Nb',
                    'Mo', 'Tc', 'Ru', 'Rh', 'Pd', 'Ag', 'Cd', 'In', 'Sn', 'Sb',
                    'Te', 'I', 'Xe', 'Cs', 'Ba', 'La', 'Ce', 'Pr', 'Nd', 'Pm',
                    'Sm', 'Eu', 'Gd', 'Tb', 'Dy', 'Ho', 'Er', 'Tm', 'Yb', 'Lu',
                    'Hf', 'Ta', 'W', 'Re', 'Os', 'Ir', 'Pt', 'Au', 'Hg', 'Tl',
                    'Pb', 'Bi', 'Po', 'At', 'Rn', 'Fr', 'Ra', 'Ac', 'Th', 'Pa',
                    'U', 'Np', 'Pu', 'Am', 'Cm', 'Bk', 'Cf', 'Es', 'Fm']

    def _eval_fit(energy, coeffs):
        """Convert log-log polynomial fit to cross-section value"""
        log_val = polyval(coeffs, log(energy))
        return exp(log_val)

    el_cross_sections = {'energy': '{0} keV'.format(energy),
                         'citation': "J. H. Scofield (1973) Lawrence Livermore"
                                     " National Laboratory "
                                     "Report No. UCRL-51326, \n"
                                     "Parametrised as log-log order 8 "
                                     "polynomial (A. J. Jackson 2018)",
                         'link': "https://doi.org/10.2172/4545040"}

    with sqlite3.connect(db_file) as con:
        for element in elements:
            cur = con.cursor()
            cur.execute('SELECT orbital, coeffs_np FROM fits WHERE Element=?;',
                        [element])
            orbitals_fits = cur.fetchall()

            el_cross_sections.update({element: {
                orb: _eval_fit(energy, np_fromstr(coeffs))
                for orb, coeffs in orbitals_fits}})
    return el_cross_sections



###############################################################################
#                                                                             #
# GALORE: Gaussian and Lorentzian broadening for simulated spectra            #
#                                                                             #
# Developed by Adam J. Jackson (2016) at University College London            #
#                                                                             #
###############################################################################
#                                                                             #
# This file is part of Galore. Galore is free software: you can redistribute  #
# it and/or modify it under the terms of the GNU General Public License as    #
# published by the Free Software Foundation, either version 3 of the License, #
# or (at your option) any later version.  This program is distributed in the  #
# hope that it will be useful, but WITHOUT ANY WARRANTY; without even the     #
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    #
# See the GNU General Public License for more details.  You should have       #
# received a copy of the GNU General Public License along with this program.  #
# If not, see <http://www.gnu.org/licenses/>.                                 #
#                                                                             #
###############################################################################

from __future__ import print_function
import os
import csv
import sys
from collections import OrderedDict
import numpy as np


def is_gpw(filename):
    """Determine whether file is GPAW calculation by checking extension"""
    return filename.split('.')[-1] == 'gpw'


def is_doscar(filename):
    """Determine whether file is a DOSCAR by checking fourth line"""

    # This doesn't break when the file is 3 lines or less; f.readline() just
    # starts returning empty strings which also fail the test.
    with open(filename, 'r') as f:
        for i in range(3):
            f.readline()
        if f.readline().strip() == 'CAR':
            return True
        else:
            return False


def is_vasp_raman(filename):
    """Determine if file is raman-sc/vasp_raman.py data by checking header"""
    with open(filename, 'r') as f:
        line = f.readline()

    return line.strip() == '# mode    freq(cm-1)    alpha    beta2    activity'


def is_csv(filename):
    """Determine whether file is CSV by checking extension"""
    return filename.split('.')[-1] == 'csv'


def is_xml(filename):
    """Determine whether file is XML by checking extension"""
    if filename.split('.')[-1] == 'gz':
        return filename.split('.')[-2] == 'xml'
    else:
        return filename.split('.')[-1] == 'xml'


def write_txt(x_values, y_values, filename="galore_output.txt", header=None):
    """Write output to a simple space-delimited file

    Args:
        x_values (iterable): Values to print in first column
        y_value (iterable): Values to print in second column
        filename (str): Path to output file, including extension. If None,
            write to standard output instead.
        header (str): Additional line to prepend to file. If None, no header
            is used.

        """

    rows = zip(x_values, y_values)
    _write_txt_rows(rows, filename=filename, header=header)


def _write_txt_rows(rows, filename=None, header=None):
    """Write rows of data to space-separated text output

    Args:
        rows (iterable): Rows to write. Rows should be a list of values.
        filename (str or None): Filename for text output. If None, write to
            standard output instead.
        header (iterable or None): Optionally add another row to the top of the
            file. Useful if rows is a generator you don't want to mess with.

    """

    def _format_line(row):
        return ' '.join(('{0:10.6e}'.format(x) for x in row)) + '\n'

    lines = map(_format_line, rows)

    if filename is not None:
        with open(filename, 'w') as f:
            if header is not None:
                f.write(header + '\n')
            f.writelines(lines)

    else:
        if header is not None:
            print(header)
        for line in lines:
            print(line, end='')


def _write_csv_rows(rows, filename=None, header=None):
    """Write rows of data to output in CSV format

    Args:
        rows (iterable): Rows to write. Rows should be a list of values.
        filename (str or None): Filename for CSV output. If None, write to
            standard output instead.
        header (iterable or None): Optionally add another row to the top of the
            file. Useful if rows is a generator you don't want to mess with.

    """

    def _write_csv(rows, f, header):
        writer = csv.writer(f, lineterminator=os.linesep)
        if header is not None:
            writer.writerow(header)
        writer.writerows(rows)

    if filename is None:
        _write_csv(rows, sys.stdout, header=header)

    else:
        with open(filename, 'w') as f:
            _write_csv(rows, f, header=header)


def write_csv(x_values, y_values, filename="galore_output.csv", header=None):
    """Write output to a simple space-delimited file

    Args:
        x_values (iterable): Values to print in first column
        y_value (iterable): Values to print in second column
        filename (str): Path to output file, including extension. If None,
            write to standard output instead.
        header (iterable): Additional line to prepend to file. If None,
            no header is used.

        """

    rows = zip(x_values, y_values)
    _write_csv_rows(rows, filename=filename, header=header)


def write_pdos(pdos_data, filename=None, filetype="txt", flipx=False):
    """Write PDOS or XPS data to CSV file

    Args:
        pdos_data (dict): Data for pdos plot in format::

                {'el1': {'energy': values, 's': values, 'p': values ...},
                 'el2': {'energy': values, 's': values, ...}, ...}

             where DOS values are 1D numpy arrays. For deterministic output,
             use ordered dictionaries!
        filename (str or None): Filename for output. If None, write to stdout
        filetype (str): Format for output; "csv" or "txt.
        flipx (bool): Negate the x-axis (i.e. energy) values to make binding
            energies
    """

    header = ['energy']
    cols = [list(pdos_data.values())[0]['energy']]
    if flipx:
        cols[0] = -cols[0]

    for el, orbitals in pdos_data.items():
        for orbital, values in orbitals.items():
            if orbital.lower() != 'energy':
                header += ['_'.join((el, orbital))]
                cols.append(values)

    data = np.array(cols).T

    total = data[:, 1:].sum(axis=1)
    data = np.insert(data, 1, total, axis=1)
    header.insert(1, 'total')

    if filetype == 'csv':
        _write_csv_rows(data, filename=filename, header=header)
    elif filetype == 'txt':
        header = ' ' + ' '.join(('{0:12s}'.format(x) for x in header))
        _write_txt_rows(data, filename=filename, header=header)
    else:
        raise ValueError('filetype "{0}" not recognised. Use "txt" or "csv".')


def read_csv(filename):
    """Read a txt file containing frequencies and intensities

    If input file contains three columns, the first column is ignored. (It is
    presumed to be a vibrational mode index.)

    Args:
        filename (str): Path to data file

    Returns:
        n x 2 Numpy array of frequencies and intensities

    """
    return read_txt(filename, delimiter=',')


def read_txt(filename, delimiter=None):
    """Read a txt file containing frequencies and intensities

    If input file contains three columns, the first column is ignored. (It is
    presumed to be a vibrational mode index.)

    Args:
        filename (str): Path to data file

    Returns:
        n x 2 Numpy array of frequencies and intensities

    """
    xy_data = np.genfromtxt(filename, comments='#', delimiter=delimiter)

    columns = np.shape(xy_data)[1]

    if columns == 2:
        return xy_data
    elif columns == 3:
        return xy_data[:, 1:]
    elif columns < 2:
        raise Exception("Not sure how to interpret {0}: "
                        "not enough columns.".format(filename))
    else:
        raise Exception("Not sure how to interpret {0}: "
                        "too many columns.".format(filename))


def read_pdos_txt(filename):
    """Read a text file containing projected density-of-states (PDOS) data

    The first row should be a header identifying the orbitals, e.g.
    "# Energy s p d f". The following rows contain the corresponding energy and
    DOS values.

    Args:
        filename (str): Path to file for import

    Returns:
        data (np.ndarray): Numpy structured array with named columns
            corresponding to input data format.
    """
    data = np.genfromtxt(filename, names=True)
    return data


def read_doscar(filename="DOSCAR"):
    """Read an x, y series of frequencies and DOS from a VASP DOSCAR file

    Args:
        filename (str): Path to DOSCAR file

    Returns:
        data (2-tuple): Tuple containing x values and y values as lists
    """
    with open(filename, 'r') as f:
        # Scroll to line 6 which contains NEDOS
        for i in range(5):
            f.readline()
        nedos = int(f.readline().split()[2])

        # Get number of fields and infer number of spin channels
        first_dos_line = f.readline().split()
        spin_channels = (len(first_dos_line) - 1) / 2
        if spin_channels == 1:

            def _tdos_from_line(line):
                return (float(line[0]), float(line[1]))
        elif spin_channels == 2:

            def _tdos_from_line(line):
                line = [float(x) for x in line]
                return (line[0], line[1] + line[2])
        else:
            raise Exception("Too many columns in DOSCAR")

        dos_pairs = (
            [_tdos_from_line(first_dos_line)] +
            [_tdos_from_line(f.readline().split()) for i in range(nedos - 1)])

        return np.array(dos_pairs)


def read_vasprun(filename='vasprun.xml'):
    """Read a VASP vasprun.xml file to obtain the density of states

    Pymatgen must be present on the system to use this method

    Args:
        filename (str): Path to vasprun.xml file

    Returns:
        data (pymatgen.electronic_structure.dos.Dos): A pymatgen Dos object
    """
    try:
        from pymatgen.io.vasp.outputs import Vasprun
        from pymatgen.electronic_structure.core import Spin
    except ImportError as e:
        e.msg = "pymatgen package neccessary to load vasprun files"
        raise

    vr = Vasprun(filename)
    band = vr.get_band_structure()
    dos = vr.complete_dos

    if band.is_metal():
        zero_point = vr.efermi
    else:
        zero_point = band.get_vbm()['energy']

    # Shift the energies so that the vbm is at 0 eV, also taking into account
    # any gaussian broadening
    dos.energies -= zero_point
    if vr.parameters['ISMEAR'] == 0 or vr.parameters['ISMEAR'] == -1:
        dos.energies -= vr.parameters['SIGMA']

    return dos


def read_gpaw_totaldos(filename, npts=50001, width=1e-3, ref='vbm'):
    """Read total DOS from GPAW with minimal broadening

    This requires GPAW to be installed and on your PYTHONPATH!

    Args:
        filename (str): Path to GPAW calculation file. This should be a .gpw
            file generated with ``calc.write('myfilename.gpw')``.
        npts (int): Number of DOS samples
        width (float): Gaussian broadening parameter applied by GPAW. Default
            is minimal so that broadening is dominated by choices in Galore.
            Beware that there is a strong interaction between this parameter
            and npts; with a small npts and small width, many energy levels
            will be missed from the DOS!
        ref (str): Reference energy for DOS. 'vbm' or 'efermi' are accepted for
            the valence-band maximum or the Fermi energy, respectively. VBM is
            determined from calculation eigenvalues and not DOS values. If set
            to None, raw values are used.
    Returns:
        data (np.ndarray): 2D array of energy and DOS values
    """
    from gpaw import GPAW
    calc = GPAW(filename)

    if ref is None:
        ref_energy = 0
    elif ref.lower() == 'vbm':
        ref_energy, _ = calc.get_homo_lumo()
    elif ref.lower() == 'efermi':
        ref_energy = calc.get_fermi_level()

    energies, dos = calc.get_dos(npts=npts, width=width)
    return np.array(list(zip(energies - ref_energy, dos)))


def read_gpaw_pdos(filename, npts=50001, width=1e-3, ref='vbm'):
    """Read orbital-projected DOS from GPAW with minimal broadening.

    This requires GPAW to be installed and on your PYTHONPATH!

    Args:
        filename (str): Path to GPAW calculation file. This should be a .gpw
            file generated with ``calc.write('myfilename.gpw')``.
        npts (int): Number of DOS samples
        width (float): Gaussian broadening parameter applied by GPAW. Default
            is minimal so that broadening is dominated by choices in Galore.
            Beware that there is a strong interaction between this parameter
            and npts; with a small npts and small width, many energy levels
            will be missed from the DOS!
        ref (str): Reference energy for DOS. 'vbm' or 'efermi' are accepted for
            the valence-band maximum or the Fermi energy, respectively. VBM is
            determined from calculation eigenvalues and not DOS values. If set
            to None, raw values are used.

    Returns:
        pdos_data (OrderedDict): PDOS data formatted as nestled OrderedDict of:
            {element: {'energy': energies, 's': densities, 'p' ... }
    """
    from gpaw import GPAW
    calc = GPAW(filename)

    if ref is None:
        ref_energy = 0
    elif ref.lower() == 'vbm':
        ref_energy, _ = calc.get_homo_lumo()
    elif ref.lower() == 'efermi':
        ref_energy = calc.get_fermi_level()

    # Set up the structure of elements and orbitals.
    # Repeated elements will leave a single entry in the dict
    proto_orbitals = OrderedDict((('energy', np.zeros(npts)),
                                  ('s', np.zeros(npts)),
                                  ('p', np.zeros(npts)),
                                  ('d', np.zeros(npts)),
                                  ('f', np.zeros(npts))))
    pdos_data = OrderedDict([(atom.symbol, proto_orbitals.copy())
                             for atom in calc.atoms])

    # Read orbital DOS, adding to collected PDOS for that element/orbital
    for atom in calc.atoms:
        for orbital in 'spdf':
            energies, dos = calc.get_orbital_ldos(atom.index, angular=orbital,
                                                  npts=npts, width=width)
            pdos_data[atom.symbol][orbital] += dos

        pdos_data[atom.symbol]['energy'] = energies - ref_energy

    # Set any zero arrays to None so they can be easily skipped over
    # This should get rid of unused orbitals; if GPAW put some density in those
    # orbitals then we should keep that evidence rather than discard it.
    for element, orbitals in pdos_data.items():
        for orbital, dos in orbitals.items():
            if orbital != 'energy' and max(dos) == 0.:
                pdos_data[element][orbital] = None

    return pdos_data

def read_vasprun_totaldos(filename='vasprun.xml'):
    """Read an x, y series of energies and DOS from a VASP vasprun.xml file

    Pymatgen must be present on the system to use this method

    Args:
        filename (str): Path to vasprun.xml file

    Returns:
        data (np.ndarray): 2D array of energy and DOS values
    """
    dos = read_vasprun(filename)

    from pymatgen.electronic_structure.core import Spin

    # sum spin up and spin down channels
    densities = dos.densities[Spin.up]
    if len(dos.densities) > 1:
        densities += dos.densities[Spin.down]

    return np.array(list(zip(dos.energies, densities)))


def read_vasprun_pdos(filename='vasprun.xml'):
    """Read a vasprun.xml containing projected density-of-states (PDOS) data

    Pymatgen must be present on the system to use this method

    Args:
        filename (str): Path to vasprun.xml file

    Returns:
        pdos_data (np.ndarray): PDOS data formatted as nestled OrderedDict of:
            {element: {'energy': energies, 's': densities, 'p' ... }
    """
    dos = read_vasprun(filename)

    from pymatgen.electronic_structure.core import Spin, OrbitalType

    pdos_data = OrderedDict()
    for element in dos.structure.symbol_set:
        pdos_data[element] = OrderedDict([('energy', dos.energies)])
        pdos = dos.get_element_spd_dos(element)
        for orb in sorted([orb.value for orb in pdos.keys()]):
            # this way we can ensure the orbitals remain in the correct order
            orbital = OrbitalType(orb)
            # sum spin up and spin down channels
            densities = pdos[orbital].densities[Spin.up]
            if len(dos.densities) > 1:
                densities += pdos[orbital].densities[Spin.down]
            pdos_data[element][orbital.name] = densities

    return pdos_data

def read_vasp_raman(filename="vasp_raman.dat"):
    """Read output file from Vasp raman simulation

    Args:
        filename (str): Path to formatted data file generated by
            https://github.com/raman-sc/VASP - Raman intensities are computed
            by following vibrational modes and calculating polarisability. The
            generated output file is named *vasp_raman.dat* but can be renamed
            if desired. The format is five space-separated columns, headed by
            ``# mode    freq(cm-1)    alpha    beta2    activity``.

    Returns:
        2-D np.array:
            Only the columns corresponding to frequency and activity are
            retained.
    """

    data = np.genfromtxt(filename)
    return data[:, [1, -1]]



"""Plotting routines with Matplotlib"""
from __future__ import division

from collections import defaultdict
from os.path import basename as path_basename
from itertools import cycle
from six import itervalues
import logging

import numpy as np
from matplotlib import pyplot as plt

from galore import auto_limits
import galore.formats

_unit_labels = {'cm': r'cm$^{-1}$',
                'cm-1': r'cm$^{-1}$',
                'thz': 'THz',
                'ev': 'eV',
                'ry': 'Ry',
                'ha': 'Ha'}
_energy_units = ('ev', 'ry', 'ha')
_frequency_units = ('cm', 'cm-1', 'thz')


def guess_xlabel(units=None, flipx=None, energy_label=None):
    """Infer a decent x-xaxis label from available information

    Args:
        units (str): Energy or frequency unit string
        flipx (bool): Is energy scale negated to form binding energy
        energy_label (str): Header from .dat file if used"""

    if units and units.lower() in _unit_labels:
        unit_label = _unit_labels[units.lower()]
    else:
        unit_label = units

    if flipx and units.lower() in _energy_units:
        xlabel = "Binding energy / " + unit_label
    elif flipx and units:
        xlabel = "-" + energy_label + " / " + unit_label
    elif flipx:
        xlabel = "Binding energy"
    elif units and energy_label is not None:
        xlabel = energy_label + " / " + unit_label
    elif units:
        xlabel = unit_label
    else:
        xlabel = ''

    return xlabel


def add_overlay(plt, overlay, overlay_scale=None, overlay_offset=0.,
                overlay_style='o', overlay_label=None):
    """Overlay data points from file over existing plot

    Args:
        plt (matplotlib.pyplot): Pyplot object with target figure/axes active
        overlay (str): Path to overlay data file
        overlay_scale (float): y-axis scale factor for overlay data. If None,
            scale to match maximum and print this value.
        overlay_offset (float): x-xaxis offset for overlay data
        overlay_style (str): Matplotlib short code for marker/line style
        overlay_label (str): Legend label for data overlay (default: filename)

    """

    if galore.formats.is_csv(overlay):
        xy_data = galore.formats.read_csv(overlay)
    else:
        xy_data = galore.formats.read_txt(overlay)

    ax = plt.gca()
    if overlay_scale is None:
        ymax = np.max(xy_data[:, 1])
        lines = ax.lines
        ymax_plot = max(max(line.get_xydata()[:, 1]) for line in lines)

        overlay_scale = ymax_plot / ymax
        logging.info("Scaling overlay intensity by {0}".format(overlay_scale))

    if overlay_label is None:
        overlay_label = path_basename(overlay)

    plt.plot(xy_data[:, 0] + overlay_offset,
             xy_data[:, 1] * overlay_scale,
             overlay_style,
             label=overlay_label)

    return plt


def plot_pdos(pdos_data, ax=None, total=True, show_orbitals=True,
              offset=0, flipx=False, **kwargs):
    """Plot a projected density of states (PDOS)

    Args:
        pdos_data (dict): Data for pdos plot in format::

                {'el1': {'energy': values, 's': values, 'p': values ...},
                 'el2': {'energy': values, 's': values, ...}, ...}

             where DOS values are 1D numpy arrays. For deterministic plots,
             use ordered dictionaries!
        ax (matplotlib.Axes): Use existing Axes object for plot. If None,
            a new figure and axes will be created.
        total (bool): Include total DOS. This is sum over all others.
            Input x-values must be consistent, no further resampling is done.
        show_orbitals (bool): Show orbital contributions. If False, they will
            not be plotted but are still used to calculate the total DOS.
        offset (float): Bias x-axis values (e.g. to account for XPS E-Fermi),
        flipx (bool): Negate x-axis values to express negative VB energies as
            positive binding energies.

    Returns:
        (matplotlib.pyplot):
            The pyplot state machine. Can be queried to access current figure
            and axes.

        """

    # Any unset kwargs will be seen as None
    kwargs = defaultdict((lambda: None), **kwargs)

    linecycler = cycle(['--'] * 6 + [':'] * 6 + ['-.'] * 6)

    max_y = 0

    if ax is None:
        plt.style.use("seaborn-colorblind")
        fig = plt.figure()
        ax = fig.add_subplot(1, 1, 1)

    tdos = np.zeros(len(next(itervalues(pdos_data))['energy']))

    for element, el_data in pdos_data.items():
        # Field 'energy' must be present, other fields are orbitals
        assert 'energy' in el_data.keys()
        if flipx:
            x_data = -el_data['energy']
        else:
            x_data = el_data['energy']

        orbitals = list(el_data.keys())
        orbitals.remove('energy')

        for orbital in orbitals:
            if total:
                tdos += el_data[orbital]
            else:
                max_y = max(max_y, max(el_data[orbital]))

            if show_orbitals:
                ax.plot(x_data, el_data[orbital],
                        label="{0}: {1}".format(element, orbital),
                        marker='', linestyle=next(linecycler))

    if total:
        max_y = max(tdos)
        ax.plot(x_data, tdos, label="Total", color='k', linestyle='-')

    # Range based on last dataset. If that's not satisfactory, it should have
    # been pruned already by kwargs['xmin'] and kwargs['xmax']
    ax.set_xlim([min(x_data), max(x_data)])

    xlabel = guess_xlabel(units=kwargs['units'], flipx=flipx,
                          energy_label=None)
    ax.set_xlabel(xlabel)

    if kwargs['ymax'] is None or kwargs['ymin'] is None:
        # Add 10% to data range if not specified
        if kwargs['ymax'] is None:
            kwargs['ymax'] = max_y * 1.1
        if kwargs['ymin'] is None:
            kwargs['ymin'] = 0
    ax.set_ylim([kwargs['ymin'], kwargs['ymax']])
    ax.set_yticklabels([''])
    ax.legend(loc='best')

    return plt


def plot_tdos(xdata, ydata, filename=None, ax=None, **kwargs):
    """Plot a total DOS (i.e. 1D dataset)


    Args:
        xdata (iterable): x-values (energy, frequency etc.)
        ydata (iterable): Corresponding y-values (DOS or measurement intensity)
        show (bool): Display plot
        filename (str): If provided, write plot to this path/filename.
        ax (matplotlib.Axes): If provided, plot onto existing Axes object. If
            None, a new Figure will be created and the pyplot instance will be
            returned.

    Returns:
        (matplotlib.pyplot):
            The pyplot state machine. Can be queried to access current figure
            and axes.

    """

    # Any unset kwargs will be seen as None
    kwargs = defaultdict((lambda: None), **kwargs)

    plt.style.use("seaborn-colorblind")

    if ax:
        pass
    else:
        fig = plt.figure()
        ax = fig.add_subplot(1, 1, 1)

    if kwargs['flipx']:
        xdata = -xdata

    ax.plot(xdata, ydata, 'r-')
    ax.set_xlim([min(xdata), max(xdata)])

    xlabel = guess_xlabel(units=kwargs['units'], flipx=kwargs['flipx'])

    ax.set_xlabel(xlabel)

    if kwargs['ymax'] is None or kwargs['ymin'] is None:
        # Add 10% to data range if not specified
        auto_ymin, auto_ymax = auto_limits(ydata, padding=0.1)
        if kwargs['ymax'] is None:
            kwargs['ymax'] = auto_ymax
        if kwargs['ymin'] is None:
            kwargs['ymin'] = 0
    ax.set_ylim([kwargs['ymin'], kwargs['ymax']])
    ax.set_yticklabels([''])

    return plt



from __future__ import division, absolute_import
import unittest
try:
    from unittest import mock
except ImportError:
    import mock
from pkg_resources import resource_filename

from galore.cross_sections import get_cross_sections
from galore.cross_sections import get_cross_sections_json
from galore.cross_sections import get_cross_sections_yeh
from galore.cross_sections import get_cross_sections_scofield


def mock_get_cross_sections_json(path):
    return (path, 'JSON')


def mock_get_cross_sections_yeh(source):
    return (source, 'YEH')


def mock_get_cross_sections_scofield(energy, elements):
    return (energy, elements, 'SCOFIELD')


class test_cross_sections_dispatch(unittest.TestCase):
    @mock.patch(
        'galore.cross_sections.get_cross_sections_json',
        side_effect=mock_get_cross_sections_json)
    def test_json_dispatch(self, mock_func):
        """Check JSON weighting files are dispatched correctly"""
        json_file = 'nonexistent.json'
        self.assertEqual(get_cross_sections(json_file),
                         (json_file, 'JSON'))
        self.assertEqual(get_cross_sections(json_file,
                                            elements=['Na']),
                         (json_file, 'JSON'))

    @mock.patch('galore.cross_sections.get_cross_sections_yeh',
                         side_effect=mock_get_cross_sections_yeh)
    def test_yeh_dispatch(self, mock_func):
        """Check tabulated weightings are dispatched correctly"""
        self.assertEqual(get_cross_sections('alka'),
                         ('alka', 'YEH'))

    @mock.patch('galore.cross_sections.get_cross_sections_scofield',
                         side_effect=mock_get_cross_sections_scofield)
    def test_scofield_dispatch(self, mock_func):
        """Check parametrised weightings are dispatched correctly"""
        self.assertEqual(get_cross_sections(42),
                         (42., None, 'SCOFIELD'))
        self.assertEqual(get_cross_sections('1.4e2',
                                            elements=['Sc', 'O']),
                         (1.4e2, ['Sc', 'O'], 'SCOFIELD'))


class test_json_data(unittest.TestCase):
    def test_json_error(self):
        """Check we can't open non-existent JSON files"""
        with self.assertRaises(IOError):
            get_cross_sections_json('non-existent.json')

    def test_json_read(self):
        """Check JSON file is opened without trouble"""
        json_file = resource_filename(__name__, 'json_test.json')
        json_data = get_cross_sections_json(json_file)
        self.assertEqual(json_data['He']['s'], 'CORRECT')


class test_yeh_data(unittest.TestCase):
    def test_xps_yeh(self):
        """Check Al k-alpha data from Yeh/Lindau"""
        Lr_cs = get_cross_sections_yeh('AlKa')['Lr']
        self.assertAlmostEqual(Lr_cs['p'], 0.001666666666666667)

        H_cs = get_cross_sections_yeh('alka')['H']
        self.assertIsNone(H_cs['f'])

    def test_ups_yeh(self):
        """Check He(II) data from Yeh/Lindau"""
        Pr_cs = get_cross_sections_yeh('He2')['Pr']
        self.assertAlmostEqual(Pr_cs['f'], 0.848)

class test_yeh_old_keys(unittest.TestCase):
    def test_yeh_old_keys(self):
        with self.assertRaises(ValueError):
            get_cross_sections('xps')
        with self.assertRaises(ValueError):
            get_cross_sections('ups')
        with self.assertRaises(ValueError):
            get_cross_sections('haxpes')
        

class test_scofield_data(unittest.TestCase):
    def setUp(self):
        self.ref_values = [('C', 's', 2, 1,
                            # Raw value from table corresponds to full
                            # shell in this case. Parametrised value
                            # is multiplied by occupancy to match.
                            1.9334e3),
                           ('Ga', 'd', 10, 5,
                            # 3/2 and 5/2 orbital energies are
                            # 1. Normalised to 1 electron
                            # 2. Combined as simple mean
                            # 3. Multiplied by total shell occupancy
                            (6.3849e1 / 4 + 9.1599e1 / 6) / 2 * 10)]

    def test_scofield_limits(self):
        """Check Scofield parametrisation refuses to extrapolate"""
        with self.assertRaises(ValueError):
            get_cross_sections_scofield(0.5, elements=['H'])
        with self.assertRaises(ValueError):
            get_cross_sections_scofield((10, 11, 0.5), elements=['H'])
        with self.assertRaises(ValueError):
            get_cross_sections_scofield(1600, elements=['H'])
        with self.assertRaises(ValueError):
            get_cross_sections_scofield((10, 11, 1600, 12), elements=['H'])

    def test_scofield_ref(self):
        """Check Scofield parametrisation: spot-check vs tables"""

        for el, orb, occ, energy, value in self.ref_values:
            cs = get_cross_sections_scofield(energy, elements=[el])
            # Check error is below 1 percent; small error is due to
            # fitting, not implementation in this part of code!
            self.assertAlmostEqual(cs[el][orb] * occ,
                                   value, delta=(value * 0.01))
